<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.html
 * @version    $Id: zmenu.php 493 2008-11-12 14:20:49Z lks $
 */

/** zpagesorter */
require_once 'zengine/classes/zpagesorter.php';

/**
 * Класс для работы с меню.
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zmenu implements Iterator, RecursiveIterator {
    /**
     * Название пункта меню
     *
     * @var string
     */
    public $name;

    /**
     * Ссылка на внутреннюю страницу без расширения
     *
     * @var string
     */
    public $uri;

    /**
     * Дата публикации
     *
     * @var string
     */
    public $publish;

    /**
     * Ссылка на внутреннюю страницу с расширением
     *
     * @var string
     */
    public $link;

    /**
     * Массив тэгов страницы
     *
     * @var array
     */
    public $tags;

    /**
     * Флаг вывода пункта меню
     *
     * @var boolean
     */
    public $nomenuitem = false;

    /**
     * Добавляет подпункт меню
     *
     * @param zpage $page
     * @return zmenu
     */
    public function appendPage (zpage $page) {
        $zmenu = new zmenu();
        $zmenu->parent = $this;
        if ($page->getProperty('nomenuitem')) {
            $zmenu->nomenuitem = true;
        }
        $zmenu->name = (string) $page->getMenu();
        $zmenu->uri = (string) $page->getPagepath();
        $zmenu->tags = $page->getTags();
        $zmenu->publish = (string) $page->getProperty('publish');
        $zmenu->setOrderProperties($page);
        $zmenu->link = $page->getLink();
        $this->_items[] = $zmenu;
        return $zmenu;
    }

    /**
     * Возвращает массив всех тэгов, собранных в меню и подпунктах меню
     *
     * @param zmenu $zmenu
     * @return array
     */
    public function extractTags (&$tags) {
        $tags = zengine::zcache()->get('tags');
        if (false === $tags) {
            $tags = array();
            $this->_extractTags($tags);
            zengine::zcache()->put('tags', $tags);
        }
    }

    /**
     * Возвращает массив всех тэгов, собранных в меню и подпунктах меню
     *
     * @param zmenu $zmenu
     * @return array
     */
    private function _extractTags (&$tags) {
        foreach ($this as $item) {
            /* @var $item zmenu */
            if ($item->publish > ZENGINE_NOW) {
                continue;
            }
            foreach ($item->tags as $tag => $tagName) {
                if (isset($tags[$tag])) {
                    ++$tags[$tag]['count'];
                    $tags[$tag]['items'][] = $item;
                } else {
                    $tags[$tag] = array(
                        'count' => 1,
                        'name' => $tagName,
                        'items' => array($item),
                    );
                }
            }
            $item->_extractTags($tags);
        }
    }


    /**
     * Возвращает хэш-код параметров, используемых при генерации меню
     *
     * @return string
     */
    public function getHash () {
    	return $this->_hash;
    }

    /**
     * Возвращает объект родительского пункта меню
     *
     * @return zmenu
     */
    public function getParent () {
    	return $this->parent;
    }


    /**
     * Возвращает правила сортировки пунктов меню
     *
     * @return zpagesorter
     */
    public function getSorter () {
        if (empty(self::$_sorter)) {
            $sortOrder = strtolower(trim(zengine::config()->get('menu.order')));
            if (empty($sortOrder)) {
                $sortOrder = 'menu asc';
            }
            self::$_sorter = new zpagesorter($sortOrder);
        }
        return self::$_sorter;
    }

    /**
     * Находит и возвращает пункт меню, ссылающийся на заданный путь
     *
     * @param string $pagepath
     * @return zmenu
     */
    public function find ($pagepath) {
    	if ($this->uri == $pagepath) {
    	    return $this;
    	}

    	foreach ($this as $submenu) {
    	    if ($submenu->find($pagepath)) {
    	        return $submenu->find($pagepath);
    	    }
    	}

    	return null;
    }

    /**
     * Устанавливает хэш-код параметров, используемых при генерации меню
     *
     * @param string $hash
     */
    public function setHash ($hash) {
        $this->_hash = $hash;
    }

    /**
     * Возвращает true, если у меню есть подпункты
     *
     * @return boolean
     */
    public function hasSubmenu () {
        foreach ($this->_items as $item) {
            // Учитываем дату публикации
            if (! ($item->publish > ZENGINE_NOW)) {
                return true;
            }
        }
    	return false;
    }

    /**
     * Возвращает true, если у текущего пункта меню есть дочерние пункты
     *
     * @return boolean
     */
    public function hasChildren () {
    	return $this->current()->hasSubmenu();
    }

    /**
     * Возвращает дочерние пункты меню (итератор)
     *
     * @return zmenu
     */
    public function getChildren () {
    	return $this->current();
    }


    /**
     * Возвращает текущий дочерний пункт меню
     *
     * @return zmenu
     */
    public function current () {
    	return $this->_items[$this->_cursor];
    }

    /**
     * Переходит к следующему дочерниму пункту меню
     */
    public function next () {
        do {
    	   ++$this->_cursor;
        } while (isset($this->_items[$this->_cursor]) && $this->_items[$this->_cursor]->publish > ZENGINE_NOW);
    }

    /**
     * Возвращает true, если текущий дочерний пункт существует
     */
    public function valid () {
    	return isset($this->_items[$this->_cursor]);
    }

    /**
     * Возвращает ключ текущего дочернего пункта меню
     */
    public function key () {
    	return $this->_cursor;
    }

    /**
     * Сбрасывает указатель на первый элемент из списка дочерних пунктов меню
     */
    public function rewind () {
        // Учитываем дату публикации
    	$this->_cursor = -1;
    	$this->next();
    }

    /**
     * Метод сортирует все подпункты меню
     *
     */
    public function sort () {
        $this->getSorter()->sort($this->_items);
    	foreach ($this as $item) {
    	    $item->sort();
    	}
    }

    /**
     * Сохраняет дополнительные свойства страницы, участвующие в сортировке
     *
     * @param zpage $page
     */
    private function setOrderProperties (zpage $page) {
        foreach ($this->getSorter()->getSortOrders() as $order => $by) {
            if (empty($this->$order)) {
                if (method_exists($page, 'get' . $order)) {
                    $this->$order = $page->{'get' . $order}();
                } else {
                    $this->$order = $page->getProperty($order);
                }
            }
        }
    }

    /**
     * Возвращает указанное свойство класса
     *
     * @param string $name Название свойства
     * @return mixed
     */
    public function getProperty ($name) {
        if (isset($this->$name)) {
    	   return $this->$name;
        }

        return null;
    }

    /**
     * Дочерние пункты меню
     *
     * @var array
     */
    private $_items = array();

    /**
     * Указатель на текущий дочерний пункт меню
     *
     * @var int
     */
    private $_cursor = 0;

    /**
     * Массив с информацией о правилах сортировки пунктов меню
     *
     * @var zpagesorter
     */
    static private $_sorter = null;

    /**
     * Ссылка на родительский пункт меню
     *
     * @var zmenu
     */
    private $parent = null;

    /**
     * Хэш параметров, которые использовались для генерации меню
     *
     * @var string
     */
    private $_hash;
}
